@@ -1,4 +1,4 @@
-/* $Id: hparser.h,v 2.28 2004/11/16 17:57:47 gisle Exp $
+/* $Id: hparser.h,v 2.30 2004/11/23 10:26:59 gisle Exp $
  *
  * Copyright 1999-2004, Gisle Aas
  * Copyright 1999-2000, Michael A. Chase
@@ -77,6 +77,7 @@ struct p_state {
     char* literal_mode;
     bool  is_cdata;
     bool  no_dash_dash_comment_end;
+    char *pending_end_tag;
 
     /* unbroken_text option needs a buffer of pending text */
     SV*    pend_text;
@@ -108,6 +109,7 @@ struct p_state {
     /* other configuration stuff */
     SV* bool_attr_val;
     struct p_handler handlers[EVENT_COUNT];
+    bool argspec_entity_decode;
 
     /* filters */
     HV* report_tags;
